@@ -17,7 +17,9 @@
 //
 // Copyright (C) 2006 Kristian Høgsberg <krh@redhat.com>
 // Copyright (C) 2006 Krzysztof Kowalczyk <kkowalczyk@gmail.com>
-// Copyright (C) 2008-2010, 2012 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2008-2010, 2012, 2014 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2012-2014 Fabio D'Urso <fabiodurso@hotmail.it>
+// Copyright (C) 2013 Jason Crain <jason@aquaticape.us>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -36,9 +38,19 @@
 #include <stdlib.h> // for NULL
 #include "gtypes.h"
 
+#ifdef __clang__
+# define GOOSTRING_FORMAT __attribute__((__annotate__("gooformat")))
+#else
+# define GOOSTRING_FORMAT
+#endif
+
 class GooString {
 public:
 
+  // a special value telling that the length of the string is not given
+  // so it must be calculated from the strings
+  static const int CALC_STRING_LEN = -1;
+
   // Create an empty string.
   GooString();
 
@@ -80,18 +92,22 @@ public:
   // - <precision> is the number of digits to the right of the decimal
   //   point (for floating point numbers)
   // - <type> is one of:
-  //     d, x, o, b -- int in decimal, hex, octal, binary
-  //     ud, ux, uo, ub -- unsigned int
-  //     ld, lx, lo, lb, uld, ulx, ulo, ulb -- long, unsigned long
-  //     lld, llx, llo, llb, ulld, ullx, ullo, ullb
+  //     d, x, X, o, b -- int in decimal, lowercase hex, uppercase hex, octal, binary
+  //     ud, ux, uX, uo, ub -- unsigned int
+  //     ld, lx, lX, lo, lb, uld, ulx, ulX, ulo, ulb -- long, unsigned long
+  //     lld, llx, llX, llo, llb, ulld, ullx, ullX, ullo, ullb
   //         -- long long, unsigned long long
-  //     f, g -- double
-  //     c -- char
+  //     f, g, gs -- floating point (float or double)
+  //         f  -- always prints trailing zeros (eg 1.0 with .2f will print 1.00)
+  //         g  -- omits trailing zeros and, if possible, the dot (eg 1.0 shows up as 1)
+  //         gs -- is like g, but treats <precision> as number of significant
+  //               digits to show (eg 0.0123 with .2gs will print 0.012)
+  //     c -- character (char, short or int)
   //     s -- string (char *)
   //     t -- GooString *
   //     w -- blank space; arg determines width
   // To get literal curly braces, use {{ or }}.
-  static GooString *format(const char *fmt, ...);
+  static GooString *format(const char *fmt, ...) GOOSTRING_FORMAT;
   static GooString *formatv(const char *fmt, va_list argList);
 
   // Destructor.
@@ -118,7 +134,7 @@ public:
   GooString *append(const char *str, int lengthA=CALC_STRING_LEN);
 
   // Append a formatted string.
-  GooString *appendf(const char *fmt, ...);
+  GooString *appendf(const char *fmt, ...) GOOSTRING_FORMAT;
   GooString *appendfv(const char *fmt, va_list argList);
 
   // Insert a character or string.
@@ -139,6 +155,9 @@ public:
   int cmp(const char *sA) const;
   int cmpN(const char *sA, int n) const;
 
+  // Return true if string ends with suffix
+  GBool endsWith(const char *suffix) const;
+
   GBool hasUnicodeMarker(void);
 
   // Sanitizes the string so that it does
@@ -156,9 +175,6 @@ private:
   // results in sizeof(GooString) be a multiple of 16.
   // 24 makes sizeof(GooString) to be 32.
   static const int STR_STATIC_SIZE = 24;
-  // a special value telling that the length of the string is not given
-  // so it must be calculated from the strings
-  static const int CALC_STRING_LEN = -1;
 
   int  roundedSize(int len);
 
@@ -170,20 +186,20 @@ private:
 #ifdef LLONG_MAX
   static void formatInt(long long x, char *buf, int bufSize,
 			GBool zeroFill, int width, int base,
-			char **p, int *len);
+			char **p, int *len, GBool upperCase = gFalse);
 #else
   static void formatInt(long x, char *buf, int bufSize,
 			GBool zeroFill, int width, int base,
-			char **p, int *len);
+			char **p, int *len, GBool upperCase = gFalse);
 #endif
 #ifdef ULLONG_MAX
   static void formatUInt(unsigned long long x, char *buf, int bufSize,
 			 GBool zeroFill, int width, int base,
-			 char **p, int *len);
+			 char **p, int *len, GBool upperCase = gFalse);
 #else
   static void formatUInt(Gulong x, char *buf, int bufSize,
 			 GBool zeroFill, int width, int base,
-			 char **p, int *len);
+			 char **p, int *len, GBool upperCase = gFalse);
 #endif
   static void formatDouble(double x, char *buf, int bufSize, int prec,
 			   GBool trim, char **p, int *len);
